<?php

if( !defined('PAWZ_INCLUDED') ) {
	die();
}
/************************************************************************
phpAce Web Zipper V1.0 2008-11-19
Copyright (c) 2008 Sam Yapp
http://www.phpace.com/scripts/web-zipper/
Support Forum: http://forum.phpace.com/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***************************************************************************/

define('PAWZ_OK',0);
define('PAWZ_ERR_FILESIZE', 1);

define('PAWZ_ZIP_FILENAME', 'pawz_temporary_zip.zip');

function pawz_load_translations()
{
	global $pawz_config;
	include dirname(__FILE__) . '/translations/default.php';
	$trans = $pawz_lang;
	if( $pawz_config['language'] != '' ) {
		include dirname(__FILE__) . '/translations/'.$pawz_config['language'].'.php';
	}
	// unlike array_merge, using + preserves numeric keys
	return $pawz_lang + $trans;
}

function pawz_translate($token, $escape = true)
{
	global $pawz_translations;
	settype($token,'string');
	if( isset($pawz_translations[$token] ) ) {
		$token = $pawz_translations[$token];
	}
	if( $escape ) {
		$token = htmlspecialchars($token);
	}
	return $token;
}

function pawz_setting($name)
{
	global $pawz_config;
	if( isset($pawz_config[$name]) ) {
		return $pawz_config[$name];
	}
	return null;
}

function pawz_add_files($upload)
{
	global $pawz_config, $pawz_errors;
	$cnt = min(count($upload['tmp_name']), $pawz_config['simultaneous_uploads']);
	$total = pawz_count_files();
	for( $i =0; $i < $cnt; $i++ ) {
		if( $total >= $pawz_config['max_files'] ) {
			$pawz_errors[] = $upload['name'][$i].': '. pawz_translate('You cannot upload any more files');
		}
		else{
			if( $upload['error'][$i] != UPLOAD_ERR_OK ) {
				switch( $upload['error'][$i] ) {
					case UPLOAD_ERR_INI_SIZE:
						$pawz_errors[] = $upload['name'][$i].': '.pawz_translate('Your file is too large');
						break;
					case UPLOAD_ERR_PARTIAL:
						$pawz_errors[] = $upload['name'][$i].': '.pawz_translate('The upload was interrupted. Please try again later');
						break;
					case UPLOAD_ERR_NO_FILE:
						break;
					case UPLOAD_ERR_NO_TMP_DIR:
					case 7:
					case 8:
						$pawz_errors[] = pawz_translate('A server error occurred. Please try again later');
						break;
					default: die($upload['error']);break;
				}
			}
			elseif( !is_uploaded_file($upload['tmp_name'][$i]) ) {
			}
			else{
				$name = $upload['name'][$i];
				$filesize = filesize($upload['tmp_name'][$i]);
				if( $filesize <= pawz_setting('max_filesize') * (1024*1024) ) {
					$dir = pawz_setting('temp_dir') . '/' . pawz_get_user_dir();
					if( preg_match('#^(.+)\.(.+)$#i', $name, $match ) ) {
						$basename = $match[1];
						$ext = $match[2];
                        $ok = true;
                        if( $pawz_config['allowed_filetypes'] != '' ) {
                            $allowed = explode(',',$pawz_config['allowed_filetypes']);
                            if( !in_array(strtolower($ext),$allowed)) {
                                $ok = false;
                            }
                        }
                        elseif( $pawz_config['banned_filetypes'] != '' ) {
                            if( in_array(strtolower($ext), explode(',',$pawz_config['banned_filetypes']))) {
                                $ok = false;
                            }
                        }
                        if( $ok ) {
    						$newname = $dir . '/' . $basename . '.' . $ext;
        					$attempts = 0;
            				while( file_exists($newname) && $attempts < 100 ) {
                				$attempts++;
                    			$newname = $dir . '/' . $basename . '_' . $attempts . '.' . $ext;
                        	}
                            if( file_exists($newname) ) {
                                $newname = $dir . '/' . $basename . '_' . mt_rand(1000,100000) . '.' . $ext;
                            }
                            move_uploaded_file($upload['tmp_name'][$i], $newname);
    						$total++;
                        }
                        else {
                            $pawz_errors[] = htmlspecialchars($name).': '.pawz_translate('Filetype not allowed');
                        }
					}
					else{
						$pawz_errors[] = htmlspecialchars($name).': '.pawz_translate('file too large.');
					}
				}
			}
		}
	}
}

function pawz_delete_zip()
{
    if( pawz_has_zip() ) {
        unlink(pawz_get_zip_filename());
    }
}

function pawz_set_zip($upload)
{
	global $pawz_config, $pawz_errors;
	if( $upload['error'] != UPLOAD_ERR_OK ) {
		switch( $upload['error'] ) {
			case UPLOAD_ERR_INI_SIZE:
				$pawz_errors[] = $upload['name'].': '.pawz_translate('Your file is too large');
				break;
			case UPLOAD_ERR_PARTIAL:
				$pawz_errors[] = $upload['name'].': '.pawz_translate('The upload was interrupted. Please try again later');
				break;
			case UPLOAD_ERR_NO_FILE:
				break;
			case UPLOAD_ERR_NO_TMP_DIR:
			case 7:
			case 8:
				$pawz_errors[] = pawz_translate('A server error occurred. Please try again later');
				break;
			default: die($upload['error']);break;
		}
	}
	elseif( !is_uploaded_file($upload['tmp_name']) ) {
	}
	else{
		$zip = pawz_get_zip_filename();
		if( $zip && file_exists($zip) ) {
			unlink($zip);
		}
		$name = $upload['name'];
		$filesize = filesize($upload['tmp_name']);
		if( $filesize <= pawz_setting('max_filesize') * (1024*1024) ) {
			$dir = pawz_setting('temp_dir') . '/' . pawz_get_user_dir();
			if( preg_match('#^(.+)\.zip$#i', $name, $match ) ) {
				$basename = $match[1];
				$newname = $dir . '/' . $basename . '.zip';
				$attempts = 0;
				while( file_exists($newname) && $attempts < 100 ) {
					$attempts++;
					$newname = $dir . '/' . $basename . '_' . $attempts . '.zip';
				}
				if( file_exists($newname) ) {
					$newname = $dir . '/' . $basename . '_' . mt_rand(1000,100000) . '.zip';
				}
				move_uploaded_file($upload['tmp_name'], $newname);
				pawz_set_zip_filename($newname);
			}
			else{
				$pawz_errors[] = pawz_translate('Your filename must end in .zip');
			}
		}
		else{
			$pawz_errors[] = htmlspecialchars($name).': '.pawz_translate('file too large.');
		}
	}
}

function pawz_get_zip_files_ext($filename)
{
	$zip = zip_open($filename);
	$files = array();
	if( is_resource($zip) ) {
		while( ($zip_entry = zip_read($zip) ) ) {
			$path = zip_entry_name($zip_entry);
			$name = pawz_trim_name($path);
			$size = zip_entry_filesize($zip_entry);
			if( $size > 0 ) {
				$files[$path] = array('size' => $size, 'name' => $name);
			}
		}
		zip_close($zip);
	}
	else{
		$pawz_errors[] = 'Unable to open zip archive';
	}
	return $files;
}

function pawz_mime_from_filename($fname)
{
	$ext = preg_replace('#^.+\.#', '', $fname);
	$mime = 'application/download';
	include dirname(__FILE__) . '/mimetypes.php';
	if( isset($mime_types[$ext]) ) {
		$mime = $mime_types[$ext];
	}
	return $mime;
}

function pawz_get_zip_file_ext($zipname, $filename)
{
	$zip = zip_open($zipname);
	if( is_resource($zip) ) {
		while( ($zip_entry = zip_read($zip) ) ) {
			$path = zip_entry_name($zip_entry);
			if( $path == $filename ) {
				$size = zip_entry_filesize($zip_entry);
				if( $size > 0 ) {
					if( zip_entry_open($zip, $zip_entry) ) {			
						$name = basename($path);
						header('Content-Type: '.pawz_mime_from_filename($filename));
						header('Content-Length: '.$size);
						header('Content-Disposition: attachment; filename="'.($name).'"');
						echo zip_entry_read($zip_entry, $size);
						zip_entry_close($zip_entry);
						zip_close($zip);
						exit();
					}
				}
			}
		}
		zip_close($zip);
	}
	else{
		$pawz_errors[] = 'Unable to open zip archive';
	}
}

function pawz_get_zip_file_pcl($zipname, $filename)
{
	global $pabi_errors;
	require_once dirname(__FILE__) . '/pclzip/pclzip.lib.php';
	$zip = new PclZip($zipname);
	$list = $zip->listContent();
	if( $list ) {
		foreach( $list as $entry ) {
			if( !$entry['folder'] && $entry['filename'] == $filename ) {
				$size = $entry['size'];
				$name = basename($filename);
				header('Content-Type: '.pawz_mime_from_filename($filename));
				header('Content-Length: '.$size);
				header('Content-Disposition: attachment; filename="'.($name).'"');
				$zip->extract(PCLZIP_OPT_BY_NAME, $filename, PCLZIP_OPT_EXTRACT_IN_OUTPUT);
				exit();
			}
		}
	}
	else{
		$pabi_errors[] = pawz_translate('Unable to read from zip archive');
	}
}

function pawz_download_file($filename)
{
	global $pawz_errors;
	$zipname = pawz_get_zip_filename();
	if( $zipname && file_exists($zipname) ) {
		if( function_exists('zip_open') ) {
			pawz_get_zip_file_ext($zipname, $filename);
		}
		else{
			pawz_get_zip_file_pcl($zipname, $filename);
		}
	}
	else{
		$pawz_errors[] = htmlspecialchars($filename).': '.pawz_translate('could not read file from zip');
	}
}

function pawz_trim_name($name)
{
	$max = pawz_setting('max_filename_display_length');
	if( strlen($name) > $max ) {
		$max2 = $max-15;
		$name = substr($name,0,10).' ... '.substr($name, strlen($name) - $max2);
	}
	return $name;
}

function pawz_get_zip_files_pcl($filename)
{
	global $pabi_errors;
	require_once dirname(__FILE__) . '/pclzip/pclzip.lib.php';
	$zip = new PclZip($filename);
	$list = $zip->listContent();
	if( $list ) {
		$files = array();
		foreach( $list as $entry ) {
			if( !$entry['folder'] ) {
				$path = $entry['filename'];
				$filesize = $entry['size'];
				$name = pawz_trim_name($path);
				$files[$path] = array('size' => $filesize, 'name' => $name);
			}
		}
	}
	else{
		$pabi_errors[] = pabi_translate('Unable to read from zip archive');
	}
	return $files;
}

function pawz_get_zip_files()
{
	$filename = pawz_get_zip_filename();
	if( $filename && file_exists($filename) ) {
		if( function_exists('zip_open') ) {
			return pawz_get_zip_files_ext($filename);
		}
		else{
			return pawz_get_zip_files_pcl($filename);
		}
	}
}

function pawz_set_zip_filename($name)
{
	$_SESSION['pawz_zip_filename'] = $name;
}

function pawz_get_zip_filename()
{
	$filename = '';
	if( isset( $_SESSION['pawz_zip_filename'] ) ) {
		$filename = $_SESSION['pawz_zip_filename'];
	}
	return $filename;
}

function pawz_get_zip_name()
{
	return pawz_zipname_from_filename(pawz_get_zip_filename());
}

function pawz_zipname_from_filename($fname)
{
	return preg_replace('#^.+[/\\\\]#', '', $fname);
}

function pawz_user_dir_name()
{
	global $pawz_config;
	if( !isset($_SESSION['pawz_user_dir']) ) {
		$dir = session_id() . time();
		$_SESSION['pawz_user_dir'] = $dir;
	}
	return $_SESSION['pawz_user_dir'];
}

/**
 * Gets the dirname (and creates the dirs if they don't exist)
 * for the current user's temporary storage
 * @return The name of the directory where user images are stored
 */
function pawz_get_user_dir()
{
	global $pawz_config;
	$dir = pawz_user_dir_name();
	$pawz_config['temp_dir'] = str_replace('/', DIRECTORY_SEPARATOR, $pawz_config['temp_dir']);
	if( !is_dir($pawz_config['temp_dir'] . DIRECTORY_SEPARATOR . $dir) ) {
		mkdir($pawz_config['temp_dir'] . '/' . $dir);
		chmod($pawz_config['temp_dir'] . '/' . $dir, $pawz_config['dir_chmod']);
	}
	return $dir;
}

/**
 * Gets the names of all uploaded files
 * @return an array of filenames
 */
function pawz_get_user_files($refresh = false)
{
	global $pawz_config;
	$files = array();
	$dir = $pawz_config['temp_dir'] . '/' . pawz_user_dir_name();
	if( file_exists($dir) ) {
		$d = dir($dir);
		while( false !== ($entry = $d->read() ) ) {
			$path = $dir . '/' . $entry;
			if( !is_dir( $path ) && $entry[0] != '.' && $entry != pawz_get_zip_name() && $entry != PAWZ_ZIP_FILENAME ) {
				$files[] = $entry;
			}
		}
		$d->close();
	}
	sort($files);
	return $files;
}

function pawz_clean_user_dir()
{
	global $pawz_config;
	$dir = $pawz_config['temp_dir'] . '/' . pawz_user_dir_name();
	if( file_exists($dir) ) {
		pawz_rmdir($dir);
	}
}

function pawz_rmdir($dir)
{
	if( !is_dir($dir) ) {
		return;
	}
	$d = dir($dir);
	if( !$d ) {
		return;
	}
	while( false !== ($entry = $d->read() ) ) {
		$path = $dir . '/' . $entry;
		if( is_file($path) ) {
			unlink($path);
		}
		elseif( $entry[0] != '.' ){
			if( is_dir($path) ) {
				pawz_rmdir($path);
			}
		}
	}
	$d->close();
	if( is_dir($dir) ) {
		rmdir($dir);
	}
}

function pawz_empty_dir($dir)
{
	$d = dir($dir);
	while( false !== ($entry = $d->read() ) ) {
		if( is_file($dir.'/'.$entry) ) {
			unlink($dir.'/'.$entry);
		}
		elseif( $entry[0] != '.' ) {
			pawz_rmdir($dir.'/'.$entry);
		}
	}
	$d->close();
}

/**
 * Count the number of files uploaded
 * @return the number of files the user has uploaded
 */
function pawz_count_files()
{
	$files = pawz_get_user_files();
	return count($files);
}

/**
 * download the files
 */
function pawz_download_zip()
{
	global $pawz_config, $pawz_filename;
	$files = pawz_get_user_files();
	if( count($files) > 0 ) {
		for( $i = 0; $i < count($files); $i++ ) {
			$files[$i] = $pawz_config['temp_dir'] . '/' . pawz_user_dir_name() . '/' . $files[$i];
		}
		$zip = $pawz_config['temp_dir'] . '/' . pawz_user_dir_name() . '/' . PAWZ_ZIP_FILENAME;

		if( file_exists($zip) ) {
			unlink($zip);
		}

		if( 0 && class_exists('ZipArchive') && PHP_VERSION > 5) {
			$z = new ZipArchive();
			include dirname(__FILE__) . '/php5zipcode.php';
		}
		else{
			require_once dirname(__FILE__) . '/pclzip/pclzip.lib.php';
			$z = new PclZip($zip);
			$list = $z->create($files ,PCLZIP_OPT_REMOVE_ALL_PATH);
			if( $list ) {
			}
			else{
			}
		}
		if( file_exists( $zip ) ) {
			header('Content-Type: application/zip');
			header('Content-Disposition: attachment; filename='.$pawz_filename);
			header('Content-Length: '.filesize($zip));
			readfile($zip);
			unlink($zip);
			exit();
		}
	}
	return null;
}

function pawz_delete_file($filename)
{
	global $pawz_config;
	$dir = $pawz_config['temp_dir'] . DIRECTORY_SEPARATOR . pawz_get_user_dir() . DIRECTORY_SEPARATOR;
	$files = pawz_get_user_files();
	if( in_array($filename, $files) ) {
		$fname = $dir . $filename;
		unlink($fname);
	}
}

function pawz_clean_old_dirs()
{
	global $pawz_config;
	$d = dir($pawz_config['temp_dir']);
	while( false !== ($entry = $d->read() ) ) {
		if( $entry[0] != '.' ) {
			$path = $pawz_config['temp_dir'] . '/' . $entry;
			if( is_dir($path) ) {
				if( filemtime($path) < time() - (60 * $pawz_config['clean_after_minutes']) ) {
					pawz_rmdir($path);
				}
			}
		}
	}
	$d->close();
}

function pawz_has_zip()
{
	$fname = pawz_get_zip_filename();
	return ($fname && file_exists($fname));
}